# This is an automatically generated file converting from the apm format

import casadi as cs

def hs063():
    # The optimal objective is (if given in):
    f_opt = 961.7151721
    x_opt = cs.DM([3.51212, 0.216988, 3.55217])
    x = cs.MX.sym('x', 3)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(3, 1)
    lbx = -cs.inf*cs.DM.ones(3, 1)
    ubx = cs.inf*cs.DM.ones(3, 1)
    g = cs.MX.zeros(2, 1)
    lbg = -cs.inf*cs.DM.ones(2, 1)
    ubg = cs.inf*cs.DM.ones(2, 1)
    
    x0[0:3]= 2
    lbx[0:3]=0
    
    lbg[0] =  56
    ubg[0] =  56
    g[0] = 8*x[0] + 14*x[1] + 7*x[2]
    lbg[1] =  25
    ubg[1] =  25
    g[1] = x[0]**2 +  x[1]**2 + x[2]**2
    obj = 1000 - x[0]**2 - 2*x[1]**2 - x[2]**2 - x[0]*x[1] - x[0]*x[2]
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
# -*- coding: utf-8 -*-
import casadi as ca
from matplotlib import pyplot as plt
import numpy as np

# Simulating a bouncing ball with DaeBuilder and event handling
# Joel Andersson, 2025

# Start with an empty DaeBuilder instance
dae = ca.DaeBuilder('bouncing_ball')

# Model variables
t = dae.add('t', 'independent')
h = dae.add('h', 'output', dict(start = 5, initial = 'exact'))
v = dae.add('v', 'output', dict(start = 0, initial = 'exact'))

# Dynamic equations
dae.eq(dae.der(h), v)
dae.eq(dae.der(v), -9.81)
dae.disp(True)

# Event dynamics: When h < 0, reinitialize v to -0.8*v
dae.when(h < 0, [dae.reinit('v', -0.8*dae.pre(v))])
dae.disp(True)

# Simulate over 7s
tgrid = np.linspace(0, 7, 100)
sim = ca.integrator('sim', 'cvodes', dae.create(), 0, tgrid,
                    dict(transition = dae.transition()))
simres = sim(x0 = dae.start(dae.x()))

# Visualize the solution
plt.figure(1)
plt.clf()
plt.plot(tgrid, simres['xf'][0, :].T)
plt.grid()
plt.show()

# Export FMU
fmu_files = dae.export_fmu()
print('Generated files: {}'.format(fmu_files))

# Compile DLL
import os
casadi_root = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
cfiles = " ".join([f for f in fmu_files if f.endswith('.c')])
sofile = dae.name() + '.so'
os.system(f'gcc --shared -fPIC -I{casadi_root}/external_packages/FMI-Standard-3.0/headers/ {cfiles} -o {sofile}')
print(f'Compiled {sofile}')

# Package into an FMU
import zipfile
fmuname = dae.name() + '.fmu'
with zipfile.ZipFile(fmuname, 'w') as fmufile:
    # Add generated files to the archive
    for f in fmu_files:
      arcname = f if f == 'modelDescription.xml' else 'sources/' + f
      fmufile.write(f, arcname = arcname)
      os.remove(f)
    # Add compile DLL to the archive (assume Linux 64 bit)
    fmufile.write(sofile, arcname = f'binaries/x86_64-linux/{sofile}')
    os.remove(sofile)
print(f'Created FMU: {fmuname}')

# Load the FMU in FMPy
try:
    import fmpy
    fmpy.dump(fmuname)
    # Simulate the generated FMU
    res = fmpy.simulate_fmu(fmuname, stop_time=7)
    import matplotlib.pyplot as plt
    plt.figure(1)
    plt.clf()
    plt.plot(res['time'], res['h'],'-', label = 'h')
    plt.xlabel('time')
    plt.legend()
    plt.grid()
    plt.show()

except ImportError as e:
   print('FMPy not installed. Skipping FMU simulation.')



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

#
# IDAS integrator
# =====================
#
# We solve a system
#   $\dot{x}(t)=f(x(t),y(t),t)$
#   $0=g(x(t),y(t),t)$

from casadi import *
from numpy import *
from pylab import *

# We solve the following simple dae system that describes
# the dynamics of a pendulum:
# x' = u, y' = v, u' = lambda * x, v' =lambda * y - g
#   s.t. x^2+y^2 = L
#
# We retain g and L as parameters
# http://en.wikipedia.org/wiki/Differential_algebraic_equation#Examples

L = SX.sym("L")
g = SX.sym("g")

# differential states

x=SX.sym("x")
y=SX.sym("y")
u=SX.sym("u")
v=SX.sym("v")

# algebraic states

lambd=SX.sym("lambda")

# All states and parameters

x_all = vertcat(x,u,y,v)
z_all = lambd
p_all = vertcat(L,g)

# the initial state of the pendulum

P_ = [5,10] # parameters
X_ = [3,-1.0/3,4,1.0/4] # differential states
XDOT_ = [-1.0/3,1147.0/240,1.0/4,-653.0/180] # state derivatives
Z_ = [1147.0/720] # algebraic state

# We construct the DAE system

ode = vertcat(u,lambd*x,v,lambd*y+g)
alg = x**2+y**2-L**2
dae = {'x':x_all, 'z':z_all, 'p':p_all, 'ode':ode, 'alg':alg}
f = Function('f', [x_all, z_all, p_all], [ode, alg], ['x', 'z', 'p'], ['ode', 'alg'])

# Let's check we have consistent initial conditions:

res = f(p=P_, x=X_, z=Z_)
print(res['ode']) # This should be same as XDOT_
print(res['alg']) # This should be all zeros


# Let's check our jacobian $\frac{dg}{dy}$:

j = jacobian(alg,lambd)
print(j)

# Note that the jacobian is not invertible: it is not of DAE-index 1
#
# This system is not solvable with idas, because it is of DAE-index 3.
# It is impossible to lambda from the last element of the residual.

# We create a DAE system solver

I = integrator('I', 'idas', dae, {'calc_ic':False, 'init_xdot':XDOT_})

# This system is not solvable with idas, because it is of DAE-index 3.
# It is impossible obtain lambda from the last element of the residual.

try:
  I(p=P_, x0=X_, z0=Z_)
except Exception as e:
  print(e)

# We construct a reworked version od the DAE (index reduced), now it is DAE-index 1

ode = vertcat(u,lambd*x)
alg = vertcat(x**2+y**2-L**2, u*x+v*y,u**2-g*y+v**2+L**2*lambd)
x_all = vertcat(x,u)
z_all = vertcat(y,v,lambd)
dae = {'x':x_all, 'z':z_all, 'p':p_all, 'ode':ode, 'alg':alg}
f = Function('f', [x_all, z_all, p_all], [ode, alg], ['x', 'z', 'p'], ['ode', 'alg'])

# the initial state of the pendulum

P_ = [5,10] # parameters
X_ = [3,-1.0/3] # differential states
XDOT_ = [-1.0/3,1147.0/240] # state derivatives
Z_ = [4,1.0/4,1147.0/720] # algebraic state

# Let's check we have consistent initial conditions:

res = f(p=P_, x=X_, z=Z_)
print(res['ode']) # This should be the same as XDOT_
print(res['alg']) # This should be all zeros

# Let's check our jacobian:

J = f.factory('J', f.name_in(), ['jac:alg:z'])
res = J(p=P_, x=X_, z=Z_)
print(array(res["jac_alg_z"]))

# $\frac{dg}{dy}$ is invertible this time.

# We create a DAE system solver

I = integrator('I', 'idas', dae, {'t0':0, 'tf':1, 'init_xdot':XDOT_})
res = I(p=P_, x0=X_, z0=Z_)
print(res['xf'])

# Possible problems
# ==================

# If you would initialize with:

P_ = [5,10] # parameters
X_ = [5,0]  # states

# You will get an error:

try:
  I(p=P_, x0=X_, z0=Z_)
except Exception as e:
  print(e)

# Although this initialisation is consistent,
# it coincides with a singular point.



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs071():
    # The optimal objective is (if given in):
    x_opt = cs.DM([1, 4.743, 3.82115, 1.37941])
    f_opt = 17.0140173
    x = cs.MX.sym('x', 4)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(4, 1)
    lbx = -cs.inf*cs.DM.ones(4, 1)
    ubx = cs.inf*cs.DM.ones(4, 1)
    g = cs.MX.zeros(2, 1)
    lbg = -cs.inf*cs.DM.ones(2, 1)
    ubg = cs.inf*cs.DM.ones(2, 1)
    
    lbx[0]=1
    ubx[0]=5
    x0[1]= 5
    lbx[1]=1
    ubx[1]=5
    x0[2]= 5
    lbx[2]=1
    ubx[2]=5
    x0[3]= 1
    lbx[3]=1
    ubx[3]=5
    
    lbg[0] = 25
    g[0] = x[0]*x[1]*x[2]*x[3]
    lbg[1] =  40
    ubg[1] =  40
    g[1] = x[0]**2 + x[1]**2 + x[2]**2 + x[3]**2
    obj = x[0]*x[3]*(x[0]+x[1]+x[2]) + x[2]
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs060():
    # The optimal objective is (if given in):
    f_opt = 0.03256820025
    x_opt = cs.DM([1.10486, 1.19667, 1.53526])
    x = cs.MX.sym('x', 3)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(3, 1)
    lbx = -cs.inf*cs.DM.ones(3, 1)
    ubx = cs.inf*cs.DM.ones(3, 1)
    g = cs.MX.zeros(1, 1)
    lbg = -cs.inf*cs.DM.ones(1, 1)
    ubg = cs.inf*cs.DM.ones(1, 1)
    
    x0[0:3]= 2
    lbx[0:3]= -10
    ubx[0:3]= 10
    
    lbg[0] = 4 + 3*cs.sqrt(2)
    ubg[0] = 4 + 3*cs.sqrt(2)
    g[0] = x[0]*(1 + x[1]**2) + x[2]**4
    obj = (x[0] - 1)**2 + (x[0] - x[1])**2 + (x[1] - x[2])**4
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
from casadi import *

T = 10. # Time horizon
N = 20 # number of control intervals

# Declare model variables
x1 = MX.sym('x1')
x2 = MX.sym('x2')
x = vertcat(x1, x2)
u = MX.sym('u')

# Model equations
xdot = vertcat((1-x2**2)*x1 - x2 + u, x1)

# Objective term
L = x1**2 + x2**2 + u**2

# Formulate discrete time dynamics
if False:
   # CVODES from the SUNDIALS suite
   dae = {'x':x, 'p':u, 'ode':xdot, 'quad':L}
   F = integrator('F', 'cvodes', dae, 0, T/N)
else:
   # Fixed step Runge-Kutta 4 integrator
   M = 4 # RK4 steps per interval
   DT = T/N/M
   f = Function('f', [x, u], [xdot, L])
   X0 = MX.sym('X0', 2)
   U = MX.sym('U')
   X = X0
   Q = 0
   for j in range(M):
       k1, k1_q = f(X, U)
       k2, k2_q = f(X + DT/2 * k1, U)
       k3, k3_q = f(X + DT/2 * k2, U)
       k4, k4_q = f(X + DT * k3, U)
       X=X+DT/6*(k1 +2*k2 +2*k3 +k4)
       Q = Q + DT/6*(k1_q + 2*k2_q + 2*k3_q + k4_q)
   F = Function('F', [X0, U], [X, Q],['x0','p'],['xf','qf'])

# Evaluate at a test point
Fk = F(x0=[0.2,0.3],p=0.4)
print(Fk['xf'])
print(Fk['qf'])

# Start with an empty NLP
w=[]
w0 = []
lbw = []
ubw = []
J = 0
g=[]
lbg = []
ubg = []

# Formulate the NLP
Xk = MX([0, 1])
for k in range(N):
    # New NLP variable for the control
    Uk = MX.sym('U_' + str(k))
    w += [Uk]
    lbw += [-1]
    ubw += [1]
    w0 += [0]

    # Integrate till the end of the interval
    Fk = F(x0=Xk, p=Uk)
    Xk = Fk['xf']
    J=J+Fk['qf']

    # Add inequality constraint
    g += [Xk[0]]
    lbg += [-.25]
    ubg += [inf]

# Create an NLP solver
prob = {'f': J, 'x': vertcat(*w), 'g': vertcat(*g)}
solver = nlpsol('solver', 'ipopt', prob)

# Solve the NLP
sol = solver(x0=w0, lbx=lbw, ubx=ubw, lbg=lbg, ubg=ubg)
w_opt = sol['x']

# Plot the solution
u_opt = w_opt
x_opt = [[0, 1]]
for k in range(N):
    Fk = F(x0=x_opt[-1], p=u_opt[k])
    x_opt += [Fk['xf'].full()]
x1_opt = vcat([r[0] for r in x_opt])
x2_opt = vcat([r[1] for r in x_opt])

tgrid = [T/N*k for k in range(N+1)]
import matplotlib.pyplot as plt
plt.figure(1)
plt.clf()
plt.plot(tgrid, x1_opt, '--')
plt.plot(tgrid, x2_opt, '-')
plt.step(tgrid, vertcat(DM.nan(1), u_opt), '-.')
plt.xlabel('t')
plt.legend(['x1','x2','u'])
plt.grid()
plt.show()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
from numpy import random, array
import unittest
from types import *
from helpers import *
import itertools
import os
import sys

warnings.filterwarnings("ignore",category=DeprecationWarning)

class ADtests(casadiTestCase):

  def setUp(self):
    x=SX.sym("x")
    y=SX.sym("y")
    z=SX.sym("z")
    w=SX.sym("w")

    out=SX(6,1)
    out[0,0]=x
    out[2,0]=x+2*y**2
    out[4,0]=x+2*y**3+3*z**4
    out[5,0]=w

    inp=SX(6,1)
    inp[0,0]=x
    inp[2,0]=y
    inp[4,0]=z
    inp[5,0]=w

    sp = Sparsity(1,6,[0, 1, 1, 2, 2, 3, 4],[0, 0, 0, 0]).T
    spT = Sparsity(6,1,[0, 4],[0, 2, 4, 5]).T

    self.sxinputs = {
       "column" : {
            "dense": [vertcat(*[x,y,z,w])],
            "sparse": [inp] }
        , "row": {
            "dense":  [vertcat(*[x,y,z,w]).T],
            "sparse": [inp.T]
       }, "matrix": {
          "dense": [c.reshape(vertcat(*[x,y,z,w]),2,2)],
          "sparse": [c.reshape(inp,3,2)]
        }
    }

    self.mxinputs = {
       "column" : {
            "dense": [MX.sym("xyzw",4,1)],
            "sparse": [MX.sym("xyzw",sp)]
        },
        "row" : {
            "dense": [MX.sym("xyzw",1,4)],
            "sparse": [MX.sym("xyzw",spT)]
        },
        "matrix": {
            "dense": [MX.sym("xyzw",2,2)],
            "sparse": [MX.sym("xyzw",c.reshape(inp,3,2).sparsity())]
        }
    }

    def temp1(xyz):
      X=MX(6,1)
      X[0,0]=xyz.nz[0]
      X[2,0]=xyz.nz[0]+2*xyz.nz[1]**2
      X[4,0]=xyz.nz[0]+2*xyz.nz[1]**3+3*xyz.nz[2]**4
      X[5,0]=xyz.nz[3]
      return [X]

    def temp2(xyz):
      X=MX(1,6)
      X[0,0]=xyz.nz[0]
      X[0,2]=xyz.nz[0]+2*xyz.nz[1]**2
      X[0,4]=xyz.nz[0]+2*xyz.nz[1]**3+3*xyz.nz[2]**4
      X[0,5]=xyz.nz[3]
      return [X]

    def testje(xyz):
      print(vertcat(*[xyz.nz[0],xyz.nz[0]+2*xyz.nz[1]**2,xyz.nz[0]+2*xyz.nz[1]**3+3*xyz.nz[2]**4,xyz.nz[3]]).shape)

    self.mxoutputs = {
       "column": {
        "dense":  lambda xyz: [vertcat(*[xyz.nz[0],xyz.nz[0]+2*xyz.nz[1]**2,xyz.nz[0]+2*xyz.nz[1]**3+3*xyz.nz[2]**4,xyz.nz[3]])],
        "sparse": temp1
        }, "row": {
        "dense": lambda xyz: [horzcat(*[xyz.nz[0],xyz.nz[0]+2*xyz.nz[1]**2,xyz.nz[0]+2*xyz.nz[1]**3+3*xyz.nz[2]**4,xyz.nz[3]])],
        "sparse": temp2
       },
       "matrix": {
          "dense": lambda xyz: [c.reshape(vertcat(*[xyz.nz[0],xyz.nz[0]+2*xyz.nz[1]**2,xyz.nz[0]+2*xyz.nz[1]**3+3*xyz.nz[2]**4,xyz.nz[3]]),(2,2))],
          "sparse": lambda xyz: [c.reshape(temp1(xyz)[0],(3,2))]
       }
    }


    self.sxoutputs = {
       "column": {
        "dense": [vertcat(*[x,x+2*y**2,x+2*y**3+3*z**4,w])],
        "sparse": [out]
        }, "row": {
          "dense":  [vertcat(*[x,x+2*y**2,x+2*y**3+3*z**4,w]).T],
          "sparse": [out.T]
      }, "matrix" : {
          "dense":  [c.reshape(vertcat(*[x,x+2*y**2,x+2*y**3+3*z**4,w]),2,2)],
          "sparse": [c.reshape(out,3,2)]
      }
    }

    self.jacobians = {
      "dense" : {
        "dense" : lambda x,y,z,w: array([[1,0,0,0],[1,4*y,0,0],[1,6*y**2,12*z**3,0],[0,0,0,1]]),
        "sparse" : lambda x,y,z,w: array([[1,0,0,0],[0,0,0,0],[1,4*y,0,0],[0,0,0,0],[1,6*y**2,12*z**3,0],[0,0,0,1]])
        }
      ,
      "sparse" : {
        "dense" : lambda x,y,z,w: array([[1,0,0,0,0,0],[1,0,4*y,0,0,0],[1,0,6*y**2,0,12*z**3,0],[0,0,0,0,0,1]]),
        "sparse" : lambda x,y,z,w:  array([[1,0,0,0,0,0],[0,0,0,0,0,0],[1,0,4*y,0,0,0],[0,0,0,0,0,0],[1,0,6*y**2,0,12*z**3,0],[0,0,0,0,0,1]])
      }
    }

  def test_SXeval_sx(self):
    n=array([1.2,2.3,7,1.4])
    for inputshape in ["column","row","matrix"]:
      for outputshape in ["column","row","matrix"]:
        for inputtype in ["dense","sparse"]:
          for outputtype in ["dense","sparse"]:
            self.message("eval_sx on SX. Input %s %s, Output %s %s" % (inputtype,inputshape,outputtype,outputshape) )
            f=Function("f", self.sxinputs[inputshape][inputtype],self.sxoutputs[outputshape][outputtype])
            f_in = DM(f.sparsity_in(0),n)
            r = f(f_in)
            J = self.jacobians[inputtype][outputtype](*n)

            seeds = [[1,0,0,0],[0,2,0,0],[1.2,4.8,7.9,4.6]]

            y = SX.sym("y",f.sparsity_in(0))

            fseeds = [DM(f.sparsity_in(0),x) for x in seeds]
            aseeds = [DM(f.sparsity_out(0),x) for x in seeds]
            res = f(y)
            fwdsens = forward([res], [y], [[x] for x in fseeds])
            adjsens = reverse([res], [y], [[x] for x in aseeds])
            fwdsens = [x[0] for x in fwdsens]
            adjsens = [x[0] for x in adjsens]

            fe = Function("fe", [y], [res])

            re = fe(f_in)

            self.checkarray(r,re)

            for sens,seed in zip(fwdsens,fseeds):
              fe = Function("fe", [y],[sens])
              re = fe(f_in)
              self.checkarray(c.vec(re),mtimes(J,c.vec(seed)),"AD")

            for sens,seed in zip(adjsens,aseeds):
              fe = Function("fe", [y],[sens])
              re = fe(f_in)
              self.checkarray(c.vec(re),mtimes(J.T,c.vec(seed)),"AD")

  def test_MXeval_mx(self):
    n=array([1.2,2.3,7,1.4])
    for inputshape in ["column","row","matrix"]:
      for outputshape in ["column","row","matrix"]:
        for inputtype in ["dense","sparse"]:
          for outputtype in ["dense","sparse"]:
            self.message("eval_mx on MX. Input %s %s, Output %s %s" % (inputtype,inputshape,outputtype,outputshape) )
            f=Function("f", self.mxinputs[inputshape][inputtype],self.mxoutputs[outputshape][outputtype](self.mxinputs[inputshape][inputtype][0]))
            f_in = DM(f.sparsity_in(0),n)
            r = f(f_in)
            J = self.jacobians[inputtype][outputtype](*n)

            seeds = [[1,0,0,0],[0,2,0,0],[1.2,4.8,7.9,4.6]]

            y = MX.sym("y",f.sparsity_in(0))

            fseeds = [DM(f.sparsity_in(0),x) for x in seeds]
            aseeds = [DM(f.sparsity_out(0),x) for x in seeds]
            res = f(y)
            fwdsens = forward([res],[y], [[x] for x in fseeds])
            adjsens = reverse([res],[y], [[x] for x in aseeds])
            fwdsens = [x[0] for x in fwdsens]
            adjsens = [x[0] for x in adjsens]

            fe = Function('fe', [y], [res])

            re = fe(f_in)

            self.checkarray(r,re)

            for sens,seed in zip(fwdsens,fseeds):
              fe = Function("fe", [y],[sens])
              re = fe(f_in)
              self.checkarray(c.vec(re),mtimes(J,c.vec(seed)),"AD")

            for sens,seed in zip(adjsens,aseeds):
              fe = Function("fe", [y],[sens])
              re = fe(f_in)
              self.checkarray(c.vec(re),mtimes(J.T,c.vec(seed)),"AD")

  @known_bug()  # Not implemented
  def test_MXeval_sx(self):
    n=array([1.2,2.3,7,1.4])
    for inputshape in ["column","row","matrix"]:
      for outputshape in ["column","row","matrix"]:
        for inputtype in ["dense","sparse"]:
          for outputtype in ["dense","sparse"]:
            self.message("eval_sx on MX. Input %s %s, Output %s %s" % (inputtype,inputshape,outputtype,outputshape) )
            f=Function("f", self.mxinputs[inputshape][inputtype],self.mxoutputs[outputshape][outputtype](self.mxinputs[inputshape][inputtype][0]))
            f_in = [0]*f.n_in();f_in[0]=n
            f_out = f(f_in)
            r = f_out[0]
            J = self.jacobians[inputtype][outputtype](*n)

            seeds = [[1,0,0,0],[0,2,0,0],[1.2,4.8,7.9,4.6]]

            y = SX.sym("y",f.sparsity_in(0))

            fseeds = [DM(f.sparsity_in(0),x) for x in seeds]
            aseeds = [DM(f.sparsity_out(0),x) for x in seeds]
            res = f(y)
            fwdsens = forward([res],[y],[[x] for x in fseeds])
            adjsens = reverse([res],[y],[[x] for x in aseeds])
            fwdsens = [x[0] for x in fwdsens]
            adjsens = [x[0] for x in adjsens]

            fe = Function("fe", [y], [res])

            fe_in = [0]*fe.n_in();fe_in[0]=n
            fe_out = fe(fe_in)

            self.checkarray(r,fe_out[0])

            for sens,seed in zip(fwdsens,fseeds):
              fe = Function("fe", [y],[sens])
              fe_in = [0]*fe.n_in();fe_in[0]=n
              fe_out = fe(fe_in)
              self.checkarray(c.vec(fe_out[0].T),mtimes(J,c.vec(seed.T)),"AD")

            for sens,seed in zip(adjsens,aseeds):
              fe = Function("fe", [y],[sens])
              fe_in = [0]*fe.n_in();fe_in[0]=n
              fe_out = fe(fe_in)
              self.checkarray(c.vec(fe_out[0].T),mtimes(J.T,c.vec(seed.T)),"AD")

  def test_MXeval_sx_reduced(self):
    n=array([1.2,2.3,7,1.4])
    for inputshape in ["column","row","matrix"]:
      for outputshape in ["column","row","matrix"]:
        for inputtype in ["dense","sparse"]:
          for outputtype in ["dense","sparse"]:
            self.message("eval_sx on MX. Input %s %s, Output %s %s" % (inputtype,inputshape,outputtype,outputshape) )
            f=Function("f", self.mxinputs[inputshape][inputtype],self.mxoutputs[outputshape][outputtype](self.mxinputs[inputshape][inputtype][0]))
            f_in = DM(f.sparsity_in(0),n)
            r = f(f_in)

            y = SX.sym("y",f.sparsity_in(0))

            res = f(y)
            fwdsens = forward([res],[y],[])
            adjsens = reverse([res],[y],[])

            fe = Function("fe", [y],[res])

            re = f(f_in)

            self.checkarray(r,re)

  def test_Jacobian(self):
    n=array([1.2,2.3,7,4.6])
    for inputshape in ["column","row","matrix"]:
      for outputshape in ["column","row","matrix"]:
        for inputtype in ["dense","sparse"]:
          for outputtype in ["dense","sparse"]:
            for mode in ["forward","reverse"]:
              self.message(" %s Jacobian on SX. Input %s %s, Output %s %s" % (mode,inputtype,inputshape,outputtype,outputshape) )
              opts = {}
              opts["ad_weight"] = 0 if mode=='forward' else 1
              opts["ad_weight_sp"] = 0 if mode=='forward' else 1
              f=Function("f", self.sxinputs[inputshape][inputtype],self.sxoutputs[outputshape][outputtype], opts)
              Jf = jacobian_old(f, 0, 0)
              J_in = DM(f.sparsity_in(0),n)
              Jout,_ = Jf(J_in)
              J = self.jacobians[inputtype][outputtype](*n)
              self.checkarray(array(Jout),J,"Jacobian\n Mode: %s\n Input: %s %s\n Output: %s %s"% (mode, inputshape, inputtype, outputshape, outputtype))

  def test_jacobianSX(self):
    n=array([1.2,2.3,7,4.6])
    for inputshape in ["column","row","matrix"]:
      for outputshape in ["column","row","matrix"]:
        for inputtype in ["dense","sparse"]:
          for outputtype in ["dense","sparse"]:
            self.message("jacobian on SX (SCT). Input %s %s, Output %s %s" % (inputtype,inputshape,outputtype,outputshape) )
            Jf=Function("Jf",
              self.sxinputs[inputshape][inputtype],
              [
                  jacobian(
                    SX(self.sxoutputs[outputshape][outputtype][0]),
                    SX(self.sxinputs[inputshape][inputtype][0])
                  )
              ]
            )
            J_in = DM(Jf.sparsity_in(0),n)
            J_out = Jf(J_in)
            J = self.jacobians[inputtype][outputtype](*n)
            self.checkarray(array(J_out),J,"jacobian")

  def test_jacsparsity(self):
    n=array([1.2,2.3,7,4.6])
    for inputshape in ["column","row","matrix"]:
      for outputshape in ["column","row","matrix"]:
        for inputtype in ["dense","sparse"]:
          for outputtype in ["dense","sparse"]:
            self.message("jacsparsity on SX. Input %s %s, Output %s %s" % (inputtype,inputshape,outputtype,outputshape) )
            f=Function("f", self.sxinputs[inputshape][inputtype],self.sxoutputs[outputshape][outputtype])
            J = self.jacobians[inputtype][outputtype](*n)
            self.checkarray(DM.ones(f.jac_sparsity(0, 0)),array(J!=0,int),"jacsparsity")

  def test_JacobianMX(self):
    n=array([1.2,2.3,7,4.6])
    for inputshape in ["column","row","matrix"]:
      for outputshape in ["column","row","matrix"]:
        for inputtype in ["dense","sparse"]:
          for outputtype in ["dense","sparse"]:
            for mode in ["forward","reverse"]:
              self.message("adj AD on MX. Input %s %s, Output %s %s" % (inputtype,inputshape,outputtype,outputshape) )
              opts = {}
              opts["ad_weight"] = 0 if mode=='forward' else 1
              opts["ad_weight_sp"] = 0 if mode=='forward' else 1
              f=Function("f", self.mxinputs[inputshape][inputtype],self.mxoutputs[outputshape][outputtype](self.mxinputs[inputshape][inputtype][0]), opts)
              Jf = jacobian_old(f, 0, 0)
              J_in = DM(f.sparsity_in(0),n)
              J_out,_ = Jf(J_in)
              J = self.jacobians[inputtype][outputtype](*n)
              self.checkarray(J_out,J,"Jacobian\n Mode: %s\n Input: %s %s\n Output: %s %s"% (mode, inputshape, inputtype, outputshape, outputtype))

  def test_jacsparsityMX(self):
    n=array([1.2,2.3,7,4.6])
    for inputshape in ["column","row","matrix"]:
      for outputshape in ["column","row","matrix"]:
        for inputtype in ["dense","sparse"]:
          for outputtype in ["dense","sparse"]:
            for mode in ["forward","reverse"]:
              self.message(" %s jacobian on MX (SCT). Input %s %s, Output %s %s" % (mode,inputtype,inputshape,outputtype,outputshape) )
              opts = {}
              opts["ad_weight"] = 0 if mode=='forward' else 1
              opts["ad_weight_sp"] = 0 if mode=='forward' else 1
              f=Function("f", self.mxinputs[inputshape][inputtype],self.mxoutputs[outputshape][outputtype](self.mxinputs[inputshape][inputtype][0]), opts)
              Jf = jacobian_old(f, 0, 0)
              J_in = DM(f.sparsity_in(0),n)
              J_out,_ = Jf(J_in)
              J = self.jacobians[inputtype][outputtype](*n)
              self.checkarray(array(J_out),J,"jacobian")
              self.checkarray(array(DM.ones(f.jac_sparsity(0, 0))),array(J!=0,int),"jacsparsity")



  def test_hessian(self):
    self.message("Jacobian chaining")
    x=SX.sym("x")
    y=SX.sym("y")
    z=SX.sym("z")
    n=array([1.2,2.3,7])
    f=Function("f", [vertcat(*[x,y,z])],[vertcat(*[x+2*y**3+3*z**4])])
    J = jacobian_old(f, 0, 0)
    m=MX.sym("m",3,1)
    JT,_ = J(m)
    JT = Function("JT", [m],[JT.T])
    JT(n)
    H = jacobian_old(JT, 0, 0)
    H(n)
    #H_out = H(H_in)

    #print array(JT_out[0])
    #print array(H_out[0])

  def test_bugshape(self):
    self.message("shape bug")
    x=SX.sym("x")
    y=SX.sym("y")

    inp=SX(5,1)
    inp[0,0]=x
    inp[3,0]=y

    f=Function("f", [inp],[vertcat(*[x+y,x,y])])
    J = jacobian_old(f, 0, 0)
    J(DM(f.sparsity_in(0),[2,7]))

    self.assertEqual(f.size1_out(0),3,"Jacobian shape bug")
    self.assertEqual(f.size2_out(0),1,"Jacobian shape bug")


  def test_bugglibc(self):
    self.message("Code that used to throw a glibc error")
    x=SX.sym("x")
    y=SX.sym("y")

    inp=SX(5,1)
    inp[0,0]=x
    inp[3,0]=y

    f=Function("f", [inp],[vertcat(*[x+y,x,y])])
    J = jacobian_old(f, 0, 0)
    J_in = DM(f.sparsity_in(0),[2,7])
    J_out,_ = J(J_in)

    f=Function("f", [inp],[vertcat(*[x+y,x,y])])
    J = jacobian_old(f, 0, 0)

  @memory_heavy()
  def test_MX(self):

    x = MX.sym("x",2)
    y = MX.sym("y",2,2)

    f1 = Function("f1", [x,y],[x+y[0,0],mtimes(y,x)])
    
    f1_noninline = Function("f1", [x,y],[x+y[0,0],mtimes(y,x)],{"never_inline":True})

    f2 = Function("f2", [x,y],[mtimes(MX.zeros(0,2),x)])

    f3 = Function("f3", [x,y],[MX.zeros(0,0),mtimes(y,x)])

    f4 = Function("f4", [x,y],[MX.zeros(0,2),mtimes(y,x)])

    ndir = 2

    in1 = [x,y]
    v1 = [DM([1.1,1.3]),DM([[0.7,1.5],[2.1,0.9]])]

    w=x[:]
    w[1]*=2

    w2=x[:]
    w2[1]*=x[0]

    ww=x[:]
    ww[[0,1]]*=x

    wwf=x[:]
    wwf[[1,0]]*=x

    wwr=x[:]
    wwr[[0,0,1,1]]*=2

    yy=y[:,:]

    yy[:,0] = x

    yy2=y[:,:]

    yy2[:,0] = x**2

    yyy=y[:,:]

    yyy[[1,0],0] = x

    yyy2=y[:,:]

    yyy2[[1,0],0] = x**2


    def remove_first(x):
      ret = DM(x)
      if ret.numel()>0:
        ret[0,0] = DM(1,1)
        return ret.sparsity()
      else:
        return ret.sparsity()

    def remove_last(x):
      ret = DM(x)
      if ret.nnz()>0:
        ret[ret.sparsity().row()[-1],ret.sparsity().get_col()[-1]] = DM(1,1)
        return ret.sparsity()
      else:
        return x

    spmods = [lambda x: x , remove_first, remove_last]

    # TODO: sparse seeding

    y_nz = sin(y)
    y_nz.nz[[2,1]] = vertcat(x[1],x[0])

    y_nzx = sin(x)
    y_nzx.nz[1] = x[0]**2

    y_nzs = sin(y)
    y_nzs.nz[y[:,0]] = vertcat(x[1],x[0])

    y_nzxs = sin(x)
    y_nzxs.nz[y[1,0]] = x[0]**2

    # nz[] double entry
    # add

    xx = horzcat(sin(x),cos(x))

    for inputs,values,out, jac, with_sx, std in [
          (in1,v1,c.sparsity_cast(x**2,sparsify(DM([[0,1],[1,0]])).sparsity()),2*c.sparsity_cast(x,sparsify(DM([[0,0],[1,0],[0,1],[0,0]])).sparsity()),True,"c89"),
          (in1,v1,f1_noninline.call([x**2,y])[1],y*2*vertcat(*[x.T,x.T]),True,"c89"),
          (in1,[v1[0],DM([[1,1.5],[0,0.9]])],xx[y[:,0],:],blockcat([[0,cos(x[1])],[cos(x[0]),0],[0,-sin(x[1])],[-sin(x[0]),0]]),False,"c99"),
          (in1,[v1[0],DM([[1,1.5],[0,0.9]])],xx[:,y[:,0]],blockcat([[-sin(x[0]),0],[0,-sin(x[1])],[cos(x[0]),0],[0,cos(x[1])]]),False,"c99"),
          (in1,[v1[0],DM([[1,1.5],[0,0.9]])],xx[y[:,0],y[:,0]],blockcat([[0,-sin(x[1])],[-sin(x[0]),0],[0,cos(x[1])],[cos(x[0]),0]]),False,"c99"),
          (in1,v1,y_nz,blockcat([[MX(1,1),MX(1,1)],[1,MX(1,1)],[MX(1,1),1],[MX(1,1),MX(1,1)]]),True,"c89"),
          (in1,v1,y_nzx,blockcat([[cos(x[0]),MX(1,1)],[2*x[0],MX(1,1)]]),True,"c89"),
          (in1,[DM([2,1]),v1[1]],y.nz[x],DM(2,2),False,"c99"),
          (in1,[v1[0],DM([[1,1.5],[0,0.9]])],sin(x).nz[y[:,0]],blockcat([[0,cos(x[1])],[cos(x[0]),0]]),False,"c99"),
          (in1,[v1[0],DM([[2,1.5],[1,0.9]])],y_nzs,blockcat([[0,0],[1,0],[0,1],[0,0]]),False,"c99"),
          (in1,[v1[0],DM([[0,1.5],[1,0.9]])],y_nzxs,blockcat([[cos(x[0]),0],[2*x[0],0]]),False,"c99"),
          (in1,[v1[0],DM([[1,1.5],[1,0.9]])],x.nz[y[:,0]],blockcat([[0,1],[0,1]]),False,"c99"),
          (in1,v1,x,DM.eye(2),True,"c89"),
          (in1,v1,x.T,DM.eye(2),True,"c89"),
          (in1,v1,x**2,2*c.diag(x),True,"c89"),
          (in1,v1,(x**2).attachAssert(True),2*c.diag(x),True,"c89"),
          (in1,v1,(x**2).T,2*c.diag(x),True,"c89"),
          (in1,v1,c.reshape(x,(1,2)),DM.eye(2),True,"c89"),
          (in1,v1,c.reshape(x**2,(1,2)),2*c.diag(x),True,"c89"),
          (in1,v1,x+y.nz[0],DM.eye(2),True,"c89"),
          (in1,v1,x+y[0,0],DM.eye(2),True,"c89"),
          (in1,v1,x+x,2*DM.eye(2),True,"c89"),
          (in1,v1,x**2+x,2*c.diag(x)+DM.eye(2),True,"c89"),
          (in1,v1,x*x,2*c.diag(x),True,"c89"),
          (in1,v1,x*y.nz[0],DM.eye(2)*y.nz[0],True,"c89"),
          (in1,v1,x*y[0,0],DM.eye(2)*y[0,0],True,"c89"),
          (in1,v1,x[0],DM.eye(2)[0,:],True,"c89"),
          (in1,v1,(x**2)[0],horzcat(*[2*x[0],MX(1,1)]),True,"c89"),
          (in1,v1,x[0]+x[1],DM.ones(1,2),True,"c89"),
          (in1,v1,sin(repmat(x**2,1,3)),repmat(cos(c.diag(x**2))*2*c.diag(x),3,1),True,"c89"),
          (in1,v1,sin(repsum((x**2).T,1,2)),cos(x[0]**2+x[1]**2)*2*x.T,True,"c89"),
          (in1,v1,vertcat(*[x[1],x[0]]),sparsify(DM([[0,1],[1,0]])),True,"c89"),
          (in1,v1,vertsplit(x,[0,1,2])[1],sparsify(DM([[0,1]])),True,"c89"),
          (in1,v1,vertcat(*[x[1]**2,x[0]**2]),blockcat([[MX(1,1),2*x[1]],[2*x[0],MX(1,1)]]),True,"c89"),
          (in1,v1,vertsplit(x**2,[0,1,2])[1],blockcat([[MX(1,1),2*x[1]]]),True,"c89"),
          (in1,v1,vertsplit(x**2,[0,1,2])[1]**3,blockcat([[MX(1,1),6*x[1]**5]]),True,"c89"),
          (in1,v1,horzcat(*[x[1],x[0]]).T,sparsify(DM([[0,1],[1,0]])),True,"c89"),
          (in1,v1,horzcat(*[x[1]**2,x[0]**2]).T,blockcat([[MX(1,1),2*x[1]],[2*x[0],MX(1,1)]]),True,"c89"),
          (in1,v1,diagcat(*[x[1]**2,y,x[0]**2]),
            blockcat(  [[MX(1,1),2*x[1]]] + ([[MX(1,1),MX(1,1)]]*14)  + [[2*x[0],MX(1,1)]] )
          ,True,"c89"),
          (in1,v1,horzcat(*[x[1]**2,x[0]**2]).T,blockcat([[MX(1,1),2*x[1]],[2*x[0],MX(1,1)]]),True,"c89"),
          (in1,v1,x[[0,1]],sparsify(DM([[1,0],[0,1]])),True,"c89"),
          (in1,v1,(x**2)[[0,1]],2*c.diag(x),True,"c89"),
          (in1,v1,x[[0,0,1,1]],sparsify(DM([[1,0],[1,0],[0,1],[0,1]])),True,"c89"),
          (in1,v1,(x**2)[[0,0,1,1]],blockcat([[2*x[0],MX(1,1)],[2*x[0],MX(1,1)],[MX(1,1),2*x[1]],[MX(1,1),2*x[1]]]),True,"c89"),
          (in1,v1,wwr,sparsify(DM([[2,0],[0,2]])),True,"c89"),
          (in1,v1,x[[1,0]],sparsify(DM([[0,1],[1,0]])),True,"c89"),
          (in1,v1,x[[1,0],0],sparsify(DM([[0,1],[1,0]])),True,"c89"),
          (in1,v1,w,sparsify(DM([[1,0],[0,2]])),True,"c89"),
          (in1,v1,w2,blockcat([[1,MX(1,1)],[x[1],x[0]]]),True,"c89"),
          (in1,v1,ww,2*c.diag(x),True,"c89"),
          (in1,v1,wwf,vertcat(*[x[[1,0]].T,x[[1,0]].T]),True,"c89"),
          (in1,v1,yy[:,0],DM.eye(2),True,"c89"),
          (in1,v1,yy2[:,0],2*c.diag(x),True,"c89"),
          (in1,v1,yyy[:,0],sparsify(DM([[0,1],[1,0]])),True,"c89"),
          (in1,v1,mtimes(y,x),y,True,"c89"),
          (in1,v1,mtimes(x.T,y.T),y,True,"c89"),
          (in1,v1,mac(y,x,DM.zeros(Sparsity.triplet(2,1,[1],[0]))),y[Sparsity.triplet(2,2,[1,1],[0,1])],True,"c89"),
          (in1,v1,mac(x.T,y.T,DM.zeros(Sparsity.triplet(2,1,[1],[0]).T)),y[Sparsity.triplet(2,2,[1,1],[0,1])],True,"c89"),
          (in1,v1,mtimes(y[Sparsity.triplet(2,2,[0,1,1],[0,0,1])],x),y[Sparsity.triplet(2,2,[0,1,1],[0,0,1])],True,"c89"),
          (in1,v1,mtimes(x.T,y[Sparsity.triplet(2,2,[0,1,1],[0,0,1])].T),y[Sparsity.triplet(2,2,[0,1,1],[0,0,1])],True,"c89"),
          (in1,v1,mtimes(y,x**2),y*2*vertcat(*[x.T,x.T]),True,"c89"),
          (in1,v1,sin(x),c.diag(cos(x)),True,"c89"),
          (in1,v1,sin(x**2),c.diag(cos(x**2)*2*x),True,"c89"),
          (in1,v1,x*y[:,0],c.diag(y[:,0]),True,"c89"),
          (in1,v1,x*y.nz[[0,1]],c.diag(y.nz[[0,1]]),True,"c89"),
          (in1,v1,x*y.nz[[1,0]],c.diag(y.nz[[1,0]]),True,"c89"),
          (in1,v1,x*y[[0,1],0],c.diag(y[[0,1],0]),True,"c89"),
          (in1,v1,x*y[[1,0],0],c.diag(y[[1,0],0]),True,"c89"),
          (in1,v1,c.dot(x,x),(2*x).T,True,"c89"),
          (in1,v1,c.dot(x**2,x),(3*x**2).T,True,"c89"),
          #(in1,v1,c.det(horzcat(*[x,DM([1,2])])),DM([-1,2])), not implemented
          (in1,v1,f1.call(in1)[1],y,True,"c89"),
          (in1,v1,f1.call([x**2,y])[1],y*2*vertcat(*[x.T,x.T]),True,"c89"),
          (in1,v1,f2.call(in1)[0],DM.zeros(0,2),True,"c89"),
          (in1,v1,f2(x**2,y),DM.zeros(0,2),True,"c89"),
          (in1,v1,f3.call(in1)[0],DM.zeros(0,2),True,"c89"),
          (in1,v1,f3.call([x**2,y])[0],DM.zeros(0,2),True,"c89"),
          (in1,v1,f4.call(in1)[0],DM.zeros(0,2),True,"c89"),
          (in1,v1,f4.call([x**2,y])[0],DM.zeros(0,2),True,"c89"),
          #(in1,v1,f1([x**2,[]])[1],DM.zeros(2,2),True,"c89"),
          #(in1,v1,f1([[],y])[1],DM.zeros(2,2),True,"c89"),
          (in1,v1,vertcat(*[x,DM(0,1)]),DM.eye(2),True,"c89"),
          (in1,v1,project(x**2, sparsify(DM([0,1])).sparsity()),blockcat([[MX(1,1),MX(1,1)],[MX(1,1),2*x[1]]]),True,"c89"),
          (in1,v1,c.dot(x,y[:,0]),y[:,0].T,True,"c89"),
          (in1,v1,x.nz[DM([[1,0]])].T*y.nz[DM([[0,2]])],blockcat([[MX(1,1),y.nz[0]],[y.nz[2],MX(1,1)]]),True,"c89"),
          (in1,v1,x.nz[c.diag([1,0])]*y.nz[c.diag([0,2])],blockcat([[MX(1,1),y.nz[0]],[MX(1,1),MX(1,1)],[MX(1,1),MX(1,1)],[y.nz[2],MX(1,1)]]),True,"c89"),

     ]:
      fun = Function("fun", inputs,[out])
      out_eval_mx = fun.call([e+1e-300 for e in inputs],True,False)[0] 
      for out in [out,out_eval_mx]:
          self.check_eval_mx(out)
          fun = Function("fun", inputs,[out,jac])

          fun_ad = [Function("fun", inputs,[out,jac], {'ad_weight':w, 'ad_weight_sp':w}) for w in [0,1]]


          fun_out = fun.call(values)


          if with_sx:
            funsx = fun.expand("expand_fun")
            funsx_ad = [f.expand('expand_'+f.name()) for f in fun_ad]
            funsx_out = funsx.call(values)
            self.checkarray(fun_out[0],funsx_out[0])
            self.checkarray(fun_out[1],funsx_out[1])
          else:
            funsx_ad = None
          if "pow" in str(out) and os.name=='nt':
            pass # Known bug #3038
          else:
            self.check_codegen(fun,inputs=values,std=std,digits=codegen_check_digits)
          self.check_serialize(fun,inputs=values)

          J_ = fun_out[1]

          def vec(l):
            ret = []
            for i in l:
              ret.extend(i)
            return ret

          storage2 = {}
          storage = {}

          vf_mx = None

          for f in [fun] + ([fun.expand('expand_'+fun.name())] if with_sx else []):
            d1 = f.forward(ndir)
            d2 = f.reverse(ndir)

            num_in = f.n_in()
            num_out = f.n_out()

            # evalThings
            for sym in [MX.sym, SX.sym]:
              if f.is_a('MXFunction') and sym==SX.sym: continue
              if f.is_a('SXFunction') and sym==MX.sym: continue

              # dense
              for spmod,spmod2 in itertools.product(spmods,repeat=2):
                fseeds = [[sym("f",spmod(f.sparsity_in(i))) for i in range(f.n_in())]  for d in range(ndir)]
                aseeds = [[sym("a",spmod2(f.sparsity_out(i)))  for i in range(f.n_out())] for d in range(ndir)]
                inputss = [sym("i",f.sparsity_in(i)) for i in range(f.n_in())]

                res = f.call(inputss,not f.is_a("SXFunction"))
                fwdsens = forward(res,inputss,fseeds,dict(always_inline=not f.is_a("SXFunction")))
                adjsens = reverse(res,inputss,aseeds,dict(always_inline=not f.is_a("SXFunction")))

                fseed = [DM(fseeds[d][0].sparsity(),random.random(fseeds[d][0].nnz())) for d in range(ndir) ]
                aseed = [DM(aseeds[d][0].sparsity(),random.random(aseeds[d][0].nnz())) for d in range(ndir) ]
                vf = Function("vf", inputss+vec([fseeds[i]+aseeds[i] for i in range(ndir)]),list(res) + vec([list(fwdsens[i])+list(adjsens[i]) for i in range(ndir)]))

                vf_in = list(values)
                offset = len(inputss)

                for d in range(ndir):
                  vf_in.append(fseed[d])
                  for i in range(len(values)-1):
                    vf_in.append(0)

                  vf_in.append(aseed[d])
                  vf_in.append(0)

                vf_out = vf.call(vf_in)
                if sym is MX.sym:
                    self.check_eval_mx([vvcat(e) for e in fwdsens])
                    self.check_eval_mx([vvcat(e) for e in adjsens])
                if "pow" in str(out) and os.name=='nt':
                  pass # Known bug #3038
                else:
                  self.check_codegen(vf,inputs=vf_in,std=std,digits=codegen_check_digits)
                self.check_serialize(vf,inputs=vf_in)

                offset = len(res)
                for d in range(ndir):
                  seed = array(fseed[d].T).ravel()
                  sens = array(vf_out[offset+0].T).ravel()
                  offset+=len(inputss)
                  self.checkarray(sens,mtimes(J_,seed),"eval Fwd %d %s" % (d,str(type(f))+str(sym)))

                  seed = array(aseed[d].T).ravel()
                  sens = array(vf_out[offset+0].T).ravel()
                  offset+=len(inputss)

                  self.checkarray(sens,mtimes(J_.T,seed),"eval Adj %d %s" % (d,str([vf_out[i] for i in range(vf.n_out())])))


                assert(offset==vf.n_out())

                # Complete random seeding
                random.seed(1)
                vf_in = []
                for i in range(vf.n_in()):
                  vf_in.append(DM(vf.sparsity_in(i),random.random(vf.nnz_in(i))))

                vf_out = vf.call(vf_in)
                if "pow" in str(out) and os.name=='nt':
                  pass # Known bug #3038
                else:
                  self.check_codegen(vf,inputs=vf_in,std=std,digits=codegen_check_digits)
                self.check_serialize(vf,inputs=vf_in)
                storagekey = (spmod,spmod2)
                if not(storagekey in storage):
                  storage[storagekey] = []
                storage[storagekey].append(vf_out)

                # Added to make sure that the same seeds are used for SX and MX
                if sym is MX.sym:
                  vf_mx = vf

              # Second order sensitivities
              for sym2 in [MX.sym, SX.sym]:

                if vf.is_a('MXFunction') and sym2==SX.sym: continue
                if vf.is_a('MXFunction') and sym2==MX.sym: continue

                for spmod_2,spmod2_2 in itertools.product(spmods,repeat=2):
                  fseeds2 = [[sym2("f",vf_mx.sparsity_in(i)) for i in range(vf.n_in())] for d in range(ndir)]
                  aseeds2 = [[sym2("a",vf_mx.sparsity_out(i))  for i in range(vf.n_out()) ] for d in range(ndir)]
                  inputss2 = [sym2("i",vf_mx.sparsity_in(i)) for i in range(vf.n_in())]

                  res2 = vf.call(inputss2,not vf.is_a("SXFunction"))
                  fwdsens2 = forward(res2,inputss2,fseeds2,dict(always_inline=not vf.is_a("SXFunction")))
                  adjsens2 = reverse(res2,inputss2,aseeds2,dict(always_inline=not vf.is_a("SXFunction")))

                  if sym2 is MX.sym:
                      self.check_eval_mx([vvcat(e) for e in fwdsens2])
                      self.check_eval_mx([vvcat(e) for e in adjsens2])

                  vf2 = Function("vf2", inputss2+vec([fseeds2[i]+aseeds2[i] for i in range(ndir)]),list(res2) + vec([list(fwdsens2[i])+list(adjsens2[i]) for i in range(ndir)]))

                  random.seed(1)
                  vf2_in = []
                  for i in range(vf2.n_in()):
                    vf2_in.append(DM(vf2.sparsity_in(i),random.random(vf2.nnz_in(i))))

                  vf2_out = vf2.call(vf2_in)
                  if "pow" in str(out) and os.name=='nt':
                    pass # Known bug #3038
                  else:
                    self.check_codegen(vf2,inputs=vf2_in,std=std,digits=codegen_check_digits)
                  self.check_serialize(vf2,inputs=vf2_in)
                  storagekey = (spmod,spmod2)
                  if not(storagekey in storage2):
                    storage2[storagekey] = []
                  storage2[storagekey].append(vf2_out)

          # Remainder of eval testing
          for store,order in [(storage,"first-order"),(storage2,"second-order")]:
            for stk,st in list(store.items()):
              for i in range(len(st)-1):
                for k,(a,b) in enumerate(zip(st[0],st[i+1])):
                  if b.numel()==0 and sparsify(a).nnz()==0: continue
                  if a.numel()==0 and sparsify(b).nnz()==0: continue
                  self.checkarray(sparsify(a),sparsify(b),("%s, output(%d)" % (order,k)))

          for expand in [False] + ([True] if with_sx else []):
            #  jacobian()
            for mode in ["forward","reverse"]:
              ind = 0 if mode=='forward' else 1
              f = funsx_ad[ind] if expand else fun_ad[ind]

              Jf = jacobian_old(f, 0, 0)
              Jf_out = Jf.call(values)

              if "pow" in str(out) and os.name=='nt':
                pass # Known bug #3038
              else:
                self.check_codegen(Jf,inputs=values,std=std,digits=codegen_check_digits)
              self.check_serialize(Jf,inputs=values)
              self.checkarray(Jf_out[0],J_)
              self.checkarray(DM.ones(Jf.sparsity_out(0)),DM.ones(J_.sparsity()),str(out)+str(mode))
              self.checkarray(DM.ones(f.jac_sparsity(0, 0)),DM.ones(J_.sparsity()))

          # Scalarized
          if out.is_empty(): continue
          s_i  = out.sparsity().row()[0]
          s_j  = out.sparsity().get_col()[0]
          s_k = s_i*out.size2()+s_j
          H_ = None

          for expand in [False] + ([True] if with_sx else []):
            for mode in ["forward","reverse"]:
              w = 0 if mode=='forward' else 1
              f = Function("fun", inputs,[out[s_i,s_j],jac[s_k,:].T], {'ad_weight':w, 'ad_weight_sp':w})
              if expand: f=f.expand('expand_'+f.name())
              f_out = f.call(values)
              J_ = f_out[1]

              Hf = hessian_old(f, 0, 0)
              Hf_out = Hf.call(values)
              if "pow" in str(out) and os.name=='nt':
                pass # Known bug #3038
              else:
                self.check_codegen(Hf,inputs=values,std=std,digits=codegen_check_digits)
              self.check_serialize(Hf,inputs=values)
              if H_ is None:
                H_ = Hf_out[0]
              self.checkarray(Hf_out[0],H_,failmessage=("mode: %s" % mode))

  def test_repmat(self):
    X = MX.sym("x",2,2)

    for e in[X.T+repmat(MX.zeros(2,1),1,2)]:
    
        for weight in [0,1]:

            F = Function('f',[vec(X)],[e],{"ad_weight_sp": weight})
            J0 = F.jac_sparsity(0,0,False)
            
            J0.spy()

            J1 = F.expand().jac_sparsity(0,0,False)
            
            J1.spy()
            
            assert J0==J1

if __name__ == '__main__':
    unittest.main()



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs065():
    # The optimal objective is (if given in):
    f_opt = 0.9535288567
    x_opt = cs.DM([3.65046, 3.65046, 4.62042])
    x = cs.MX.sym('x', 3)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(3, 1)
    lbx = -cs.inf*cs.DM.ones(3, 1)
    ubx = cs.inf*cs.DM.ones(3, 1)
    g = cs.MX.zeros(1, 1)
    lbg = -cs.inf*cs.DM.ones(1, 1)
    ubg = cs.inf*cs.DM.ones(1, 1)
    
    x0[0]= -5
    x0[1]=  5
    x0[2]=  0
    
    ubg[0] = 48
    g[0] = x[0]**2 + x[1]**2 + x[2]**2
    lbx[0] = -4.5
    ubx[0] =  4.5
    lbx[1] = -4.5
    ubx[1] =  4.5
    lbx[2] = -5
    ubx[2] =    5
    obj = (x[0] - x[1])**2 + (x[0] + x[1] - 10)**2/9 + (x[2] - 5)**2
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs008():
    # The optimal objective is (if given in):
    f_opt = -1
    x = cs.MX.sym('x', 2)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(2, 1)
    lbx = -cs.inf*cs.DM.ones(2, 1)
    ubx = cs.inf*cs.DM.ones(2, 1)
    g = cs.MX.zeros(2, 1)
    lbg = -cs.inf*cs.DM.ones(2, 1)
    ubg = cs.inf*cs.DM.ones(2, 1)
    
    x0[0]= 2
    x0[1]= 1
    
    lbg[0] =  25
    ubg[0] =  25
    g[0] = x[0]**2 + x[1]**2
    lbg[1] =  9
    ubg[1] =  9
    g[1] = x[0]*x[1]
    obj = -1
    
    a = cs.sqrt((25 + cs.sqrt(301))/2)
    b = cs.sqrt((25 - cs.sqrt(301))/2)
    x_opt = cs.DM([a, 9/a])
    
    f = obj
    print('hs008: There are more feasible points: (-a, -9/a), (b, 9/b), (-b, -9/b)')
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
import unittest
from types import *
from helpers import *
from casadi.tools import *

class Toolstests(casadiTestCase):

  @known_bug()  # Currently failing
  def test_structure(self):


    s = struct(['x','y','z'])

    print(s)

    with self.assertRaises(Exception):
      struct_symSX(['x','x','z'])
    with self.assertRaises(Exception):
      struct_symSX(['x','y','z','y'])



    s = struct_symSX([('x','y'),'z'])
    self.assertEqual(s.size,3)

    init = s(0)

    init['x'] = 5
    init['y'] = 7
    init['z'] = 8

    x,y,z = s[...]

    self.checkarray(init.cat,DM([5,7,8]))

    init[...] = [1,2,3]

    self.checkarray(init.cat,DM([1,2,3]))

    init = s()
    init['x'] = 5
    init['y'] = 7
    init['z'] = 8

    self.checkarray(init.cat,DM([5,7,8]))

    init[{}] = {'x': 12}
    self.checkarray(init.cat,DM([12,7,8]))

    init = s(inf)
    self.checkarray(init.cat,DM([inf,inf,inf]))

    f = DM.zeros(3,1)
    init = s(f)
    f[0] = 5
    self.checkarray(init.cat,DM([5,0,0]))

    s = struct_symSX(['x','y','z'],order=['x','y','z'])
    with self.assertRaises(Exception):
      s = struct_symSX([('x','y'),'z'],order=['x','y','z'])

    s = struct_symSX(['x','y','z'])
    self.assertEqual(s.size,3)
    self.assertEqual(s[vertcat,['x','y']].shape[0],2)
    self.assertTrue(isinstance(s[list,vertcat,['x','y']],list))

    s = struct_symSX([entry('x',repeat=5),entry('y',repeat=6),entry('z')])
    self.assertEqual(s.size,12)
    self.assertEqual(len(s["x"]),5)
    self.assertEqual(len(s["y"]),6)
    self.assertTrue(s.cat.nz[1].name().startswith("x"))
    s = struct_symSX([(entry('x',repeat=5),entry('y',repeat=[6,5])),entry('z')])


    self.checkarray(s.f["x"],[0,6,12,18,24])
    self.checkarray(s.f["z"],[35])
    self.checkarray(s.f["y"],[1,2,3,4,5,7,8,9,10,11,13,14,15,16,17,19,20,21,22,23,25,26,27,28,29,30,31,32,33,34])

    self.assertEqual(s.size,36)
    self.assertEqual(len(s["x"]),5)
    self.assertEqual(len(s["y"]),6)
    self.assertTrue(s.cat.nz[1].name().startswith("y"))
    s = struct_symSX([entry("x",shape=(3,2)),entry("y",shape=2),entry("z",shape=Sparsity.dense(3)),entry("w",shape=Sparsity.upper(5))])
    self.assertEqual(s.size,6+2+3+15)
    self.assertTrue(s["x"].sparsity()==Sparsity.dense(3,2))
    self.assertTrue(s["y"].sparsity()==Sparsity.dense(2,1))
    self.assertTrue(s["z"].sparsity()==Sparsity.dense(3,1))
    self.assertTrue(s["w"].sparsity()==Sparsity.upper(5))

    x  = SX.sym("x",2)
    x2 = SX.sym("x2",2)
    s = struct_symSX([entry('a',sym=x),'y','z'])
    self.assertTrue(s.cat.nz[0].name().startswith("x"))
    self.assertEqual(s.size,4)
    with self.assertRaises(Exception):
      struct_symSX([entry('a',sym=x+x),'y','z'])
    with self.assertRaises(Exception):
      struct_symSX([entry('a',sym=[x+x]),'y','z'])
    s = struct_symSX([entry('a',sym=vertcat(*[x,x2])),'y','z'])
    self.assertEqual(s.size,6)
    with self.assertRaises(Exception):
      s = struct_symSX([entry('a',repeat=6,sym=x),'y','z'])

    s = struct_symSX(['x','y','z'])

    S = struct_symSX([entry("X",sym=s)])
    self.assertTrue(S.cat.nz[0].name()=="x")
    S = struct_symSX([entry("X",struct=s)])
    self.assertTrue(S.cat.nz[0].name()=="X_x")
    S = struct_symSX([entry("X",repeat=[5],struct=s)])
    self.assertEqual(S.size,15)

    s0 = struct_symSX(['x','y','z'])
    s1 = struct_symSX(['x','y'])
    S0 = struct_symSX([entry("X",struct=s0),entry("Y",repeat=[5],struct=s0),entry("Z",struct=s0)])
    S1 = struct_symSX([entry("X",struct=s1),entry("Y",repeat=[5],struct=s1)])
    num0 = S0(0)
    num1 = S1(1)

    num0[nesteddict] = num1[nesteddict]

    S = struct_symSX([entry("P",shapestruct=(s0,s0))])

    num = S(0)
    num["P"] = DM([[1,2,3],[4,5,6],[7,8,9]])
    self.checkarray(num["P",index["x"],index["y"]],DM([2]))
    self.checkarray(num["P",index["x"],:],DM([1,2,3]))
    self.checkarray(num["P",:,index["y"]],DM([2,5,8]))
    self.checkarray(num["P",:,:],DM([[1,2,3],[4,5,6],[7,8,9]]))

    self.checkarray(num["P",indexf[["x","y"]],indexf[["z","x"]]],DM([[3,1],[6,4]]))

    self.checkarray(num["P",index[list,vertcat,["x","y"]],index[list,vertcat,["z","x"]]],DM([[3,1],[6,4]]))

    S = struct_symSX([entry("P",shapestruct=s0)])

    num = S(0)
    num["P"] = DM([1,2,3])
    self.checkarray(num["P",index["x"]],DM([1]))
    self.checkarray(num["P",:],DM([1,2,3]))

    self.checkarray(num["P",index["x"],0],DM([1]))
    self.checkarray(num["P",:,0],DM([1,2,3]))

    with self.assertRaises(Exception):
      num["P",:,index["x"]]


    S = struct_symSX([entry("P",shapestruct=(3,s0))])

    num = S(0)
    num["P"] = DM([[1,2,3],[4,5,6],[7,8,9]])
    with self.assertRaises(Exception):
      self.checkarray(num["P",index["x"],index["y"]],DM([2]))
    with self.assertRaises(Exception):
      self.checkarray(num["P",index["x"],:],DM([1,2,3]).T)
    self.checkarray(num["P",:,index["y"]],DM([2,5,8]))
    self.checkarray(num["P",:,:],DM([[1,2,3],[4,5,6],[7,8,9]]))

    self.checkarray(num["P",:,indexf[["z","x"]]],DM([[3,1],[6,4],[9,7]]))

    self.checkarray(num["P",:,index[list,vertcat,["z","x"]]],DM([[3,1],[6,4],[9,7]]))

    S = struct_symSX([entry("P",shapestruct=s0)])



    s0 = struct_symSX(['x','y',entry('q',shape=4),'z'])

    S = struct_symSX([entry("P",shapestruct=(s0,s0))])

    num = S(0)
    num["P",indexf[["x","q"]],indexf[["x","q"]]] = 1

    self.checkarray(num["P"][[1,6],[1,6]],DM.zeros(2,2))
    self.checkarray(num["P"][[0,2,3,4,5],[0,2,3,4,5]],DM.ones(5,5))

    num = S(0)
    num["P",["x","q"],["x","q"]] = 1

    self.checkarray(num["P"][[1,6],[1,6]],DM.zeros(2,2))
    self.checkarray(num["P"][[0,2,3,4,5],[0,2,3,4,5]],DM.ones(5,5))

    num = S(0)
    num["P",["x","q"],["x","q"]] = DM.ones(5,5)

    self.checkarray(num["P"][[1,6],[1,6]],DM.zeros(2,2))
    self.checkarray(num["P"][[0,2,3,4,5],[0,2,3,4,5]],DM.ones(5,5))

    with self.assertRaises(Exception):
      struct_symMX(['x','x','z'])
    with self.assertRaises(Exception):
      struct_symMX(['x','y','z','y'])

    s = struct_symMX([('x','y'),'z'])
    s = struct_symMX(['x','y','z'],order=['x','y','z'])
    with self.assertRaises(Exception):
      s = struct_symMX([('x','y'),'z'],order=['x','y','z'])

    s = struct_symMX(['x','y','z'])
    self.assertEqual(s.size,3)
    s = struct_symMX([entry('x',repeat=5),entry('y',repeat=6),entry('z')])
    self.assertEqual(s.size,12)
    self.assertEqual(len(s["x"]),5)
    self.assertEqual(len(s["y"]),6)
    s = struct_symMX([(entry('x',repeat=5),entry('y',repeat=[6,5])),entry('z')])
    self.assertEqual(s.size,36)
    self.assertEqual(len(s["x"]),5)
    self.assertEqual(len(s["y"]),6)


    s = struct_symMX([entry("x",shape=(3,2)),entry("y",shape=2),entry("z",shape=Sparsity.dense(3)),entry("w",shape=Sparsity.upper(5))])
    self.assertEqual(s.size,6+2+3+15)
    self.assertTrue(s["x"].sparsity()==Sparsity.dense(3,2))
    self.assertTrue(s["y"].sparsity()==Sparsity.dense(2,1))
    self.assertTrue(s["z"].sparsity()==Sparsity.dense(3,1))
    self.assertTrue(s["w"].sparsity()==Sparsity.upper(5))

    x  = MX.sym("x",2)
    x2 = MX.sym("x2",2)
    with self.assertRaises(Exception):
      s = struct_symMX([entry('x',sym=x),'y','z'])
    with self.assertRaises(Exception):
      struct_symMX([entry('x',sym=x+x),'y','z'])
    with self.assertRaises(Exception):
      struct_symMX([entry('x',sym=[x+x]),'y','z'])
    with self.assertRaises(Exception):
      struct_symMX([entry('x',sym=vertcat(*[x,x2])),'y','z'])
    with self.assertRaises(Exception):
      s = struct_,MX.sym([(2,'x',[x,x2]),'y','z'])

    s = struct_symMX(['x','y','z'])
    S = struct_symMX([entry("X",struct=s)])
    S = struct_symMX([entry("X",repeat=[5],struct=s)])

    x = SX.sym("x",2)
    y0 = sin(x)
    y1 = cos(x)

    v = struct_SX([
      entry("x",expr=x),
      entry("y",expr=[y0,y1]),
      entry("w",expr=[[x,x**2],[x**3,x**4]])
    ])


    x = MX.sym("x",2)
    m = struct_symMX(['a','b'])
    y0 = sin(x)
    y1 = cos(x)

    V = struct_MX([
      entry("x",expr=x),
      (entry("y",expr=[y0,y1],struct=m),
      entry("w",expr=[[x,x**2],[x**3,x**4]],struct=m)
      )
    ])
    self.assertEqual(V.size,14)

    self.assertTrue(isinstance(V.cat,MX))
    def is_equalV(a,b):
      ft = Function("ft", [x,m],[a-b])
      ft_in = [0]*ft.n_in()
      for i in range(ft.n_in()):
        ft_in[i]=numpy.random.rand(*ft.size_in(i))
      ft_out = ft(ft_in)
      self.checkarray(ft_out[0],DM.zeros(*ft.size_out(0)))

    is_equalV(V["x"],x)
    is_equalV(V["y",0],y0)
    is_equalV(V["y",1],y1)
    self.assertEqual(V["y",0,'a'].shape,(1,1))

    with self.assertRaises(Exception):
      V["y",0] = MX.sym("x",4) # shape mismatch
    abc = MX.sym("abc",2)
    V["y",0] = abc

    def is_equalV(a,b):
      ft = Function("ft", [x,m,abc],[a-b])
      ft_in = [0]*ft.n_in()
      for i in range(ft.n_in()):
        ft_in[i]=numpy.random.rand(*ft.size_in(i))
      ft_out = ft(ft_in)
      self.checkarray(ft_out[0],DM.zeros(*ft.size_out(0)))

    is_equalV(V["y",0],abc)

    states = struct_symSX([
                entry('x'),
                entry('y'),
                entry('z'),
                entry('u',shape=Sparsity.dense(4)),
                entry('v',repeat=[4,2]),
                entry('w',repeat=[6]),
                entry('p',repeat=[9],shape=Sparsity.dense(6))
             ],order=['x','y','z','u',('v','w'),'p'])

    shooting = struct_symSX([entry('X',struct=states,repeat=[4,5]),entry('U',repeat=[3])],order=[('X','U')])



    self.assertEqual(shooting.size,1503)
    s = shooting["X",:,:,{}]
    self.assertTrue(isinstance(s,list))
    self.assertEqual(len(s),4)
    self.assertTrue(isinstance(s[0],list))
    self.assertEqual(len(s[0]),5)
    self.assertTrue(isinstance(s[0][0],dict))
    self.assertTrue('x' in s[0][0])
    self.assertEqual(len(s[0][0]),7)
    if GlobalOptions.getSimplificationOnTheFly():
      self.assertTrue(is_equal(s[0][0]["x"],shooting["X",0,0,"x"]))


    init = shooting(nan)

    init['X',0,-1,'p',0] = 2
    self.checkarray(init.cat[shooting.i['X',0,-1,'p',0]],DM.ones(6)*2)
    self.assertEqual(sum([i!=i for i in init.cat.nonzeros()]),1503-6)

    init['X',0,-1,'p',0] = [3]*6
    self.checkarray(init.cat[shooting.i['X',0,-1,'p',0]],DM.ones(6)*3)
    self.assertEqual(sum([i!=i for i in init.cat.nonzeros()]),1503-6)

    init['X',0,-1,'p',0] = DM([4]*6)
    self.checkarray(init.cat[shooting.i['X',0,-1,'p',0]],DM.ones(6)*4)
    self.assertEqual(sum([i!=i for i in init.cat.nonzeros()]),1503-6)

    init['X',0,-1,'p',:] = 7

    self.checkarray(init.cat[shooting.i['X',0,-1,'p',1]],DM.ones(6)*7)
    self.assertEqual(sum([i!=i for i in init.cat.nonzeros()]),1503-6*9)

    self.checkarray(init['X',0,-1,'p',horzcat,:],DM.ones(6,9)*7)

    with self.assertRaises(Exception):
      init['X',0,-1,'p',:] = [1,2,3,4,5,6]

    init['X',0,-1,'p',:] = repeated([1,2,3,4,5,6])
    self.checkarray(init['X',0,-1,'p',vertcat,:,2],DM.ones(9)*3)

    init = shooting(DM(list(range(shooting.size))))

    self.checkarray(init['X',vertcat,:,horzcat,:],init['X',blockcat,:,:])

    init = shooting(nan)

    init['X',:,:,['x','y']] = repeated(repeated([6,5]))

    print(init.cat)

    init['X',:,:,{}] = repeated(repeated({'x': 9,'y': 3}))

    print(init.cat)

    V = struct_SX(shooting)

    V['X',:,:,['x','y']] = repeated(repeated([6,5]))

    print(V)

    V = struct_symMX(shooting)

    print(V)

    V = struct_MX(shooting)

    print(V)


    init = shooting(nan)



    init['X',0,-1,'p',horzcat,:] = DM.ones(6,9)*2

    self.assertEqual(sum(init.cat==2),6*9)

    init['X',0,-1,'p',vertcat,:,2] = DM.ones(9)*3

    self.assertEqual(sum(init.cat==2),6*9-9)
    self.assertEqual(sum(init.cat==3),9)

    self.checkarray(init['X',0,-1,'p',horzcat,:,[2,3]],vertcat(*[DM.ones(1,9)*3,DM.ones(1,9)*2]))

    init['X',:,-1,'p',horzcat,:] = repeated(DM.ones(6,9)*2)

    self.assertEqual(sum(init.cat==2),4*6*9)

    init['X',0,0,'v',blockcat] = DM.ones(4,2)*7

    self.assertEqual(sum(init.cat==7),4*2)

    init['X',:,0,'v',blockcat] = repeated(DM.ones(4,2)*7)

    self.assertEqual(sum(init.cat==7),4*2*4)

    init['X',:,:,'v',blockcat] = repeated(DM.ones(4,2)*7)
    self.assertEqual(sum(init.cat==7),4*2*4*5)

    init['X',0,0,'p',:] = list(range(9))


    print(index["a"])

    init = shooting(list(range(shooting.size)))
    for i in range(shooting.size):
      ci = shooting.getCanonicalIndex(i)
      self.assertEqual(i, init.__getitem__(ci))
      self.assertTrue("_".join(map(str,ci)).startswith(shooting.cat.nz[i].name()))

  def test_structure_prefix(self):
    self.message("structure prefix")
    s = struct(["x","y","z"])

    S = struct_symSX([entry("X",repeat=12,struct=s)])

    print(S.__class__)
    print(S.prefix)

    a = S.prefix["X"]

    num = S()

    init = num.prefix["X",-1]

    init["x"] = 12

    self.assertEqual(num["X",-1,"x"],init["x"])

    self.checkarray(num.cat,DM([0]*(S.size-3) + [12,0,0]))

  def test_structure_repeated_dmatrix(self):
    self.message("repeated dmatrix")
    s = struct(["x","y","z"])
    d = DM.zeros(s.size,12)
    a = s.repeated(d)

    a[:,"x"] = list(range(12))

    self.checkarray(a[4,"x"],DM([4]))
    self.checkarray(d,vertcat(*[DM(list(range(12))).T,DM.zeros(1,12),DM.zeros(1,12)]))

  def test_structure_squared_dmatrix(self):
    self.message("squared dmatrix")
    s = struct(["x","y","z"])
    d = DM.zeros(3,3)
    a = s.squared(d)

    a["x","y"] = 2
    a["y","x"] = 1

    self.checkarray(a["x","y"],DM([2]))
    self.checkarray(a["y","x"],DM([1]))

    self.checkarray(d,DM([[0,2,0],[1,0,0],[0,0,0]]))

  def test_structure_squared_repeated_dmatrix(self):
    self.message("squared repeated dmatrix")
    s = struct(["x","y","z"])

    d = DM([[0,2,0,0,0,0,0,2,0],[1,0,0,0,0,0,1,0,0],[0,0,0,0,0,0,0,0,0]])
    a = s.squared_repeated(d)
    self.checkarray(a[0,"x","y"],DM([2]))
    self.checkarray(a[0,"y","x"],DM([1]))
    self.checkarray(a[-1,"x","y"],DM([2]))
    self.checkarray(a[-1,"y","x"],DM([1]))

    d = DM.zeros(3,9)
    a = s.squared_repeated(d)

    a[0,"x","y"] = 2
    a[0,"y","x"] = 1
    a[-1,"x","y"] = 2
    a[-1,"y","x"] = 1
    self.checkarray(d,DM([[0,2,0,0,0,0,0,2,0],[1,0,0,0,0,0,1,0,0],[0,0,0,0,0,0,0,0,0]]))

  def test_typemaps(self):
    self.message("typemaps")
    s = struct(["x","y","z"])
    d = DM.zeros(3,3)
    a = s.squared(d)

    print(type(a))
    print(sin(a))
    if not sys.version_info >= (3, 0):
      print(a+1)

  def test_sparse(self):
    a = struct_symSX([entry("a",shape=Sparsity.diag(5))])
    b = struct_symMX([(entry("b",struct=a))])

    self.checkarray(b["b"].shape,(5,1))
    self.checkarray(b["b","a"].shape,(5,5))

  def test_symm(self):
    a = struct_symSX([entry("P",shape=(3,3),type='symm')])

    b = a()
    b["P"] = DM([[0,3,6],[1,4,7],[2,5,8]])

    self.assertEqual(a.size,6)
    self.checkarray(a["P"].shape,(3,3))

    f = Function("f", [a],[a["P"]])
    f_out = f(list(range(6)))

    self.checkarray(f_out,DM([[0,1,3],[1,2,4],[3,4,5]]))
    self.checkarray(b["P"],DM([[0,3,6],[3,4,7],[6,7,8]]))
    self.checkarray(b.cat,DM([0,3,4,6,7,8]))

    states = struct_symSX(["x","y","z"])

    a = struct_symSX([entry("P",shapestruct=(states,states),type='symm')])

    b = a()
    b["P"] = DM([[0,3,6],[1,4,7],[2,5,8]])

    self.checkarray(b["P",["x","z"],["x","z"]] ,DM([[0,6],[6,8]]))

    b["P","y","x"] = 66
    self.checkarray(b["P"],DM([[0,66,6],[66,4,7],[6,7,8]]))


    b["P","x","y"] = 666

    self.checkarray(b["P"],DM([[0,666,6],[666,4,7],[6,7,8]]))

    b["P",["x"],["y","z"]] = DM([1,12]).T
    self.checkarray(b["P"],DM([[0,1,12],[1,4,7],[12,7,8]]))

    b["P",["x","z"],["x","z"]] = DM([[11,0],[0,11]])
    self.checkarray(b["P"],DM([[11,1,0],[1,4,7],[0,7,11]]))

    a = struct_symSX([entry("P",shape=Sparsity.banded(3,1),type='symm')])

    b = a()
    with self.assertRaises(Exception):
      b["P"] = DM([[11,1,2],[1,4,5],[2,5,8]])

    with self.assertRaises(Exception):
      b["P"] = sparsify(DM([[11,0,1],[1,4,5],[0,5,8]]))

    b["P"] = sparsify(DM([[11,1,0],[1,4,5],[0,5,8]]))

    self.checkarray(b["P"],DM([[11,1,0],[1,4,5],[0,5,8]]))

    with self.assertRaises(Exception):
      b["P",:,0] = DM([1,2,3])

    with self.assertRaises(Exception):
      b["P",0,:] = DM([1,2,3]).T

    b["P",:,0] = sparsify(DM([1,2,0]))
    self.checkarray(b["P"],DM([[1,2,0],[2,4,5],[0,5,8]]))

    b["P",0,:] = sparsify(DM([11,12,0])).T
    self.checkarray(b["P"],DM([[11,12,0],[12,4,5],[0,5,8]]))

  def test_callableExtraIndex(self):
    a = struct_symSX([entry("a",shape=(5,3)),entry("b",shape=(4,3))])
    b = a()

    b["a",vec] = list(range(15))
    self.checkarray(b.cat,DM(list(range(15))+[0]*12))

    self.checkarray(b["a",vec],DM(list(range(15))))

    b["a",vec] = list(range(15))

    self.checkarray(b["a",vec],DM(list(range(15))))

  @unittest.skipIf(sys.version_info >= (3, 0),"too lazy to fix now")
  def test_pickling(self):
    import pickle

    x = struct([entry("a",shape=(2,3)),entry("b",shape=(2,3))])

    s = pickle.dumps(x)

    w = pickle.loads(s)

    self.assertEqual(x.size,w.size)
    x = struct([entry("a",shape=(2,3)),entry("b",shape=(2,3))])

    y = x()

    y["b",:,2] = 12

    s = pickle.dumps(y)

    w = pickle.loads(s)

    self.checkarray(w["b",:,2],DM([12,12]))

  def test_numpyint(self):
    state = struct_symSX(['x', 'y'])
    x = struct_symSX([entry('states', struct=state, repeat=10)])
    x_init = x()
    x_init['states', 0, 'x'] # OK
    a = [1,2,3]
    x_init['states', int32(0), 'x']
    x_init['states', int64(0), 'x']

  def test_numpyint(self):
    s = struct_symSX(list(map(entry, 'xyz'))) # OK
    print(s['x'])
    s = struct_symSX(list(map(entry, 'xyz'))) # IndexError: list index out of range
    print(s['x'])

  def test_pickling_null(self):
    import pickle
    s = struct_symMX([
      entry("a",shape=(2,3)),
      entry("b",shape=(0,0))
    ])

    tt = pickle.dumps(s)

    print(pickle.loads(tt))

  def test_bug_structSXMX(self):
    n= 2
    x_sx = struct_symSX([
        entry("x",shape=n),
        entry("S",shape=(n,n))
    ])

    x_mx = struct_symSX([
        entry("x",shape=n),
        entry("S",shape=(n,n))
    ])

    X_sx = struct_SX(x_sx)
    X_sx["x"] = DM(list(range(n)))
    X_sx["S"] = c.reshape(DM(list(range(n,n+n*n))),n,n)

    X_mx = struct_MX(x_sx)
    X_mx["x"] = DM(list(range(n)))
    X_mx["S"] = c.reshape(DM(list(range(n,n+n*n))),n,n)

    self.checkarray(x_sx.struct.map[("S",)],c.reshape(DM(list(range(n,n+n*n))),n,n))
    self.checkarray(x_mx.struct.map[("S",)],c.reshape(DM(list(range(n,n+n*n))),n,n))
    self.checkarray(X_sx.cat,DM(list(range(n+n*n))))
    self.checkarray(X_mx.cat,DM(list(range(n+n*n))))

    for s, S in [(x_sx,struct_symSX),(x_mx,struct_symMX)]:
      h = S([entry("w",struct=s)])
      hX = struct_SX(h)
      hX["w","x"] = DM(list(range(n)))
      hX["w","S"] = c.reshape(DM(list(range(n,n+n*n))),n,n)

      self.checkarray(h.struct.map[("w","S",)],c.reshape(DM(list(range(n,n+n*n))),n,n))
      self.checkarray(hX.cat,DM(list(range(n+n*n))))

      self.checkarray(h.struct.map[("w",)],DM(list(range(n+n*n))))
      self.checkarray(hX["w"],DM(list(range(n+n*n))))

      hX["w"] = DM(list(range(n+n*n)))
      self.checkarray(hX.cat,DM(list(range(n+n*n))))

    n= 2
    m = 3
    x_sx = struct_symSX([
        entry("x",shape=n),
        entry("S",shape=(n,m))
    ])

    x_mx = struct_symSX([
        entry("x",shape=n),
        entry("S",shape=(n,m))
    ])

    X_sx = struct_SX(x_sx)
    X_sx["x"] = DM(list(range(n)))
    X_sx["S"] = c.reshape(DM(list(range(n,n+n*m))),n,m)

    X_mx = struct_MX(x_sx)
    X_mx["x"] = DM(list(range(n)))
    X_mx["S"] = c.reshape(DM(list(range(n,n+n*m))),n,m)

    self.checkarray(x_sx.struct.map[("S",)],c.reshape(DM(list(range(n,n+n*m))),n,m))
    self.checkarray(x_mx.struct.map[("S",)],c.reshape(DM(list(range(n,n+n*m))),n,m))
    self.checkarray(X_sx.cat,DM(list(range(n+n*m))))
    self.checkarray(X_mx.cat,DM(list(range(n+n*m))))

    n = 3
    x_sx = struct_symSX([
        entry("x",shape=n),
        entry("S",shape=Sparsity.upper(n))
    ])

    x_mx = struct_symSX([
        entry("x",shape=n),
        entry("S",shape=Sparsity.upper(n))
    ])

    X_sx = struct_SX(x_sx)
    X_sx["x"] = DM(list(range(n)))
    X_sx["S"] = DM(Sparsity.upper(n),list(range(n,int(n+n*(n+1)/2))))

    X_mx = struct_MX(x_sx)
    X_mx["x"] = DM(list(range(n)))
    X_mx["S"] = DM(Sparsity.upper(n),list(range(n,int(n+n*(n+1)/2))))

    self.checkarray(x_sx.struct.map[("S",)],DM(Sparsity.upper(n),list(range(n,int(n+n*(n+1)/2)))))
    self.checkarray(x_mx.struct.map[("S",)],DM(Sparsity.upper(n),list(range(n,int(n+n*(n+1)/2)))))
    self.checkarray(X_sx.cat,DM(list(range(int(n+n*(n+1)/2)))))
    self.checkarray(X_mx.cat,DM(list(range(int(n+n*(n+1)/2)))))

  def test_MX_result(self):
    s = struct_symMX(["a",entry("b",shape=2),entry("c",shape=(2,2))])

    V = s.cat

    d = s(V)

    print(d["b"])

    f = Function("f", [V],[d["a"],d["b"],d["c"]])

    s_ = s()
    s_["a"] = 1
    s_["b"] = DM([2,3])
    s_["c"] = DM([[4,5],[6,7]])
    f_out = f(s_)

    self.checkarray(f_out[0],s_["a"])
    self.checkarray(f_out[1],s_["b"])
    self.checkarray(f_out[2],s_["c"])

  def test_issue1116(self):
    S = struct([entry("A",shape=(4,4),type="symm")])
    v = S()
    v["A"]  = np.eye(4)

    a = DM(v["A"])
    v["A"]  = DM(np.eye(4))
    b = DM(v["A"])

    self.checkarray(a,b)

  @known_bug()  # Currently failing
  def test_jacobian(self):
    states = struct_symSX(["x","y"])
    controls = struct_symSX(["u","v","w"])

    #   u v w
    # x 1 3 5
    # y 2 4 6

    for t in [SX, MX]:


      J = t.sym("J",states.size,controls.size)

      J = states.product(controls,J)

      f = Function("f", [J],[J["x","v"], J["x",:] , J["y",["v","w"]],  J[:,"u"] ])
      f_in = [0]*f.n_in();f_in[0]=DM(f.sparsity_in(0),list(range(1,7)))

      f_out = f(f_in)

      self.checkarray(f_out[0],DM([3]))
      self.checkarray(f_out[1],DM([1,3,5]).T)
      self.checkarray(f_out[2],DM([4,6]).T)
      self.checkarray(f_out[3],DM([1,2]))

  def test_empty_bug(self):

    Params = struct_symMX([ ])
    params = Params()

    self.checkarray(params.shape,(0,1))

  def test_empty_expr_bug(self):

    eq = MX.sym("X")

    g = struct_MX([ entry( 'equality', expr = eq),
                    entry( 'inequality', expr = [] )   ])

    self.checkarray(g.shape,(1,1))

    self.assertTrue(len(g["inequality"])==0)
    
  def test_shape_IM(self):
    M = 1   # If M is changed to something > 1, the error disappears.
    N = 5
    s = struct_symMX([entry('u', shape = (M, N))])
    u = s.prefix['u']

    self.assertTrue(u[:, :].shape,(M, N))
    
  def test_external_transform(self):
    x = SX.sym("x")
    f = Function("f",[x],[x**2])
    
    libcasadi = CasadiMeta.shared_library_prefix()+"casadi"

    with self.assertOutputs(["Doing","bar"],["Warning"]):
        g = external_transform(libcasadi,"external_transform_test_success",f, {"foo": "bar"})
    self.checkfunction(f,g,inputs=[3])
    
    with self.assertInException("nonexistant"):
        external_transform(libcasadi,"external_transform_test_nonexistant",f, {"foo": "bar"})
    
    with self.assertInException("Cannot load shared library"):
        external_transform("libfoo","external_transform_test_success",f, {"foo": "bar"})
        
    with self.assertInException("failed"):
        external_transform(libcasadi,"external_transform_test_fail",f, {"foo": "bar"})
        
  def test_external_transform_options(self):
    libcasadi = CasadiMeta.shared_library_prefix()+"casadi"
    for x in [SX.sym("x"),MX.sym("x")]:
        with self.assertOutputs(["Doing"],["Warning"]):
            f = Function("f",[x],[x**2],{"external_transform": [[libcasadi, "external_transform_test_success"]]})
        with self.assertOutputs(["Doing","hey"],["Warning"]):
            f = Function("f",[x],[x**2],{"external_transform": [[libcasadi, "external_transform_test_success", {"hey": 123}]]})

if __name__ == '__main__':
    unittest.main()

